@@ -1,13 +1,14 @@
 import os
 from datetime import datetime
 from enum import Enum
+from typing import List
 
 import pandas as pd
 from fuzzywuzzy import fuzz
 from groclient import GroClient
 
 from grocropclient import crop_budgets
-from grocropclient.constants import FrequencyList, OtherItemsList, SourceList, MetricList, CropList, GRO_YIELD_MODELS, AreaWeightingSeriesList, ItemNameCategoryList, CROP_BUDGET_REGIONS
+from grocropclient.constants import FrequencyList, OtherItemsList, SourceList, MetricList, CropList, GRO_YIELD_MODELS, AreaWeightingSeriesList, ItemNameCategoryList, CROP_BUDGET_REGIONS, NATIONAL_AND_STATE_CROP_BUDGET_REGIONS, US_States, US_STATES_SET, US_COUNTIES_SET
 
 
 API_HOST = "api.gro-intelligence.com"
@@ -27,6 +28,11 @@ def is_valid_selection(dataseries: dict) -> bool:
     dataseries['metric_id'] in AVAILABLE_METRICS and \
     dataseries['item_id'] in AVAILABLE_ITEMS
 
+def is_valid_area_weighting_region(region_id: int) -> bool:
+    """Returns True if the selected region is USA, US state, or US county"""
+    if region_id == US_States.UNITED_STATES or region_id in US_STATES_SET or region_id in US_COUNTIES_SET:
+        return True
+    return False
 
 def days_ago(gro_date: str):
     """Return the number of days between today's data and a Gro date."""
@@ -71,6 +77,7 @@ def is_corn_or_soybeans(item_name: str) -> str:
         elif ItemNameCategoryList.SOYBEANS.value in word.lower():
             return ItemNameCategoryList.SOYBEANS.value
 
+
 class GroCropClient(object):
 
     def __init__(self, api_host=API_HOST, access_token=None, proxy_host=None, proxy_port=None,
@@ -205,10 +212,17 @@ class GroCropClient(object):
         """
         region_id = get_region_id(region_name, source_name)
         crop = is_corn_or_soybeans(item_name)
+        region_name = region_name.lower()
         if crop == ItemNameCategoryList.CORN.value:
-            yield_dict = self._client.get_area_weighted_series(AreaWeightingSeriesList.CORN_YIELD.value, ['NASS Corn (US only)'], region_id, latest_date_only=latest_date_only)
+            if region_name in NATIONAL_AND_STATE_CROP_BUDGET_REGIONS: 
+                return self.get_yield_model_df(region_name, CropList.CORN.value, 284, latest_date_only)
+            else:
+                yield_dict = self._client.get_area_weighted_series(AreaWeightingSeriesList.CORN_YIELD.value, ['NASS Corn (US only)'], region_id, latest_date_only=latest_date_only)
         elif crop == ItemNameCategoryList.SOYBEANS.value:
-            yield_dict = self._client.get_area_weighted_series(AreaWeightingSeriesList.SOYBEANS_YIELD.value, ['NASS Soybeans (US only)'], region_id, latest_date_only=latest_date_only)
+            if region_name in NATIONAL_AND_STATE_CROP_BUDGET_REGIONS:
+                return self.get_yield_model_df(region_name, CropList.SOYBEANS.value, 287, latest_date_only)
+            else:
+                yield_dict = self._client.get_area_weighted_series(AreaWeightingSeriesList.SOYBEANS_YIELD.value, ['NASS Soybeans (US only)'], region_id, latest_date_only=latest_date_only)
  
         return pd.DataFrame({'Yield': yield_dict.values()}, yield_dict.keys())
     
@@ -595,7 +609,7 @@ class GroCropClient(object):
             # We need to ask for revisions to get all daily predictions.
             # We need to ask for metadata to get the confidence interval value.
             params = { "show_revisions": True, "show_metadata": True,
-                       "source_id": SourceList.GRO_YIELD_MODEL, "frequency_id": 9,
+                       "source_id": SourceList.GRO_YIELD_MODEL.value, "frequency_id": 9,
                        "item_id":  ym["item_id"], "region_id": ym["region_id"], "metric_id": ym["metric_id"]}
         if start_date:
             params["start_date"] = start_date
@@ -624,6 +638,37 @@ class GroCropClient(object):
                 summary.append(k_summary)
         summary = summary[::-1]
         return summary
+    
+    
+    def get_yield_model_df(self, region_name: str, item_id: int, unit_id: int, latest_date_only: bool) -> pd.DataFrame:
+        """
+        Helper function that converts result of get_data_points() into a DataFrame for the Yield Model source
+        
+        returns:
+                        Yield
+        2018-05-18  49.454534
+        2018-05-19  49.423135
+        2018-06-29  54.235872
+        """
+        region_id = NATIONAL_AND_STATE_CROP_BUDGET_REGIONS[region_name].value
+        params = {
+            'source_id': SourceList.GRO_YIELD_MODEL.value,
+            'frequency_id': FrequencyList.ANNUAL.value,
+            'item_id':  item_id,
+            'region_id': region_id,
+            'metric_id': MetricList.YIELD.value,
+            'unit_id': unit_id,
+            'start_date': '2001-01-01',
+            'show_revisions': True
+        }
+        data_points = self.get_data_points(**params)
+        values = [data_point['value'] for data_point in data_points]
+        dates = [data_point['reporting_date'].split('T')[0] for data_point in data_points]
+        yield_df = pd.DataFrame(data={'Yield': values}, index=dates)
+        if latest_date_only:
+            return yield_df.tail(1)
+        else:
+            return yield_df
 
     ############
     ## Prices ##
@@ -806,22 +851,11 @@ class GroCropClient(object):
              "source_id": SourceList.USDA_NASS_CROPS,
              "frequency_id": 9,
              'start_date': start_date,
-             'end_date': end_date
+             'end_date': end_date,
+             'unit_id': 41
              }
         return self.get_data_points(**params)
 
-    def get_area_harvested_GroForecasts(self, crop: CropList, region, start_date = None, end_date = None):
-        """Returns area harvested for a crop and a region."""
-        params = { "metric_id": MetricList.AREA_HARVESTED,
-             "item_id": crop,
-             "region_id": region,
-             "source_id": SourceList.GRO_FORECASTS,
-             "frequency_id": 9,
-             'start_date': start_date,
-             'end_date': end_date
-             }
-        return self.get_data_points(**params)
-  
     def get_area_planted_USDA(self, crop: CropList, region, start_date = None, end_date = None):
         """Returns area planted for a crop and a region."""
         params = { "metric_id": MetricList.AREA_PLANTED,
@@ -830,7 +864,8 @@ class GroCropClient(object):
              "source_id": SourceList.USDA_NASS_CROPS if crop in [CropList.CORN, CropList.SOYBEANS] else SourceList.USDA_FSA,
              "frequency_id": 9,
              'start_date': start_date,
-             'end_date': end_date
+             'end_date': end_date,
+             'unit_id': 41
              }
         return self.get_data_points(**params)
 
@@ -839,10 +874,11 @@ class GroCropClient(object):
         params = { "metric_id": MetricList.AREA_PLANTED,
              "item_id": crop,
              "region_id": region,
-             "source_id": SourceList.GRO_ENTERPRISE,
+             "source_id": SourceList.GRO_ENTERPRISE_MODELS,
              "frequency_id": 9,
              'start_date': start_date,
-             'end_date': end_date
+             'end_date': end_date,
+             'unit_id': 41
              }
         return self.get_data_points(**params)
 
@@ -854,7 +890,8 @@ class GroCropClient(object):
              "source_id": SourceList.USDA_FSA,
              "frequency_id": 9,
              'start_date': start_date,
-             'end_date': end_date
+             'end_date': end_date,
+             'unit_id': 41
              }
         return self.get_data_points(**params)
 
@@ -863,10 +900,11 @@ class GroCropClient(object):
         params = { "metric_id": MetricList.AREA_PREVENTED_NON_IRRIGATED if non_irrigated else MetricList.AREA_PREVENTED,
              "item_id": crop.value,
              "region_id": region,
-             "source_id": SourceList.GRO_ENTERPRISE,
+             "source_id": SourceList.GRO_ENTERPRISE_MODELS,
              "frequency_id": 9,
              'start_date': start_date,
-             'end_date': end_date
+             'end_date': end_date,
+             'unit_id': 41
              }
         return self.get_data_points(**params)
 
@@ -915,7 +953,7 @@ class GroCropClient(object):
     #############
     ## Climate ##
     #############
-    
+   
     def get_Gro_Drought_Index(self, region_id, frequency = FrequencyList.DAILY.value, start_date=None):
         """Returns Drought Index for a region and a frequency {daily, weekly, monthly} (use FrequencyList enum)."""
         if isinstance(frequency, Enum):
@@ -984,3 +1022,68 @@ class GroCropClient(object):
         if start_date:
             params["start_date"] = start_date
         return self.get_data_points(**params)
+
+    ####################
+    ## Area weighting ##
+    ####################
+
+    def get_area_weighting_series_names(self):
+        """
+        Returns a list of valid series names that can be used to
+        form the inputs of `get_area_weighted_series`.
+        """
+        return self._client.get_area_weighting_series_names()
+    
+    def get_area_weighting_weight_names(self):
+        """
+        Returns a list of valid weight names that can be used to
+        form the inputs of `get_area_weighted_series`.
+        """
+        return self._client.get_area_weighting_weight_names()
+
+    def get_area_weighted_series(self, series_name, weight_names, region_id, method='sum', latest_date_only=False) -> dict:
+        """
+        Compute weighted average on selected series with the given weights.
+        Returns a dictionary mapping dates to weighted values.
+
+        Parameters
+        ----------
+        series_name: str
+        The series we want to weight. For example, for land surface temperature this would be `LST_daily`. Call `get_area_weighting_series_names` for all possible choices
+        weight_names: Union[str, List[str]]
+        The weights by which we want to weight the data. Can be a single weight, like `'Corn'`, or a list of weights, like `['Corn', 'Soybeans']`
+        region_id: Union[int, List[int]]
+        The region(s) for which we want to weight data. Can be a single region, like `1215` for the United States, or a list of regions, like `[13066, 13064]` for Iowa and Illinois
+        method: str
+        Can be `'sum'` or `'normalize'`. Only matters when there are multiple weights. `'sum'` will add weights together if possible for each district. `'normalize'` will adjust weights so that each has an equal effect on the output
+        latest_date_only: bool
+        Set to `True` if you only want the latest weighted data point. Set to `False` if you want the full history
+
+        Returns 
+        -------
+        A Python dictionary with this format:
+
+        ```
+        {'2003-01-31': 1.326309,
+         '2003-02-01': 1.32882,
+         ... 
+         '2023-01-22': 1.040599, 
+         '2023-01-23': 1.050879}
+
+        ```
+        """
+        # Make sure region_id is always a list
+        if type(region_id) == int:
+            region_id: list = [region_id]
+
+        # Validate that regions are in the allowed regions
+        for r in region_id:
+            if not is_valid_area_weighting_region(r):
+                raise Exception(f"{r} is not the United States, a US state, or a US county")
+
+        # All regions have been checked, do the weighted average
+        return self._client.get_area_weighted_series(series_name = series_name,
+                                                     weight_names = weight_names, 
+                                                     region_id = region_id, 
+                                                     method = method, 
+                                                     latest_date_only = latest_date_only)
